from os import listdir  # 导入获取目录列表的方法
from pclient import random_string
from pserver import Node, UNHANDLED
from xmlrpc.client import ServerProxy, Fault
from threading import Thread
from time import sleep
from os.path import abspath  # 导入获取绝对路径的方法
import wx
import sys
import os  # 导入os模块

HEAD_START = 0.1
SECRET_LENGTH = 10

class ListableNode(Node):  # 定义Node子类
    def file_list(self):  # 定义获取文件列表的方法
        return listdir(self.dirname)  # 返回文件列表

class Client(wx.App):
    def __init__(self, url_file, dir_name, url):
        self.dir_name = dir_name
        self.secret = random_string(SECRET_LENGTH)
        node = ListableNode(url, dir_name, self.secret)  # 实例化ListableNode节点对象
        thread = Thread(target=node._start)
        thread.setDaemon(1)
        thread.start()
        sleep(HEAD_START)
        self.server = ServerProxy(url)
        for line in open(url_file):
            self.server.hello(line.strip())
        super(Client, self).__init__()  # 这一句放在最后，否则Client对象的服务器代理尚未创建，无法正常运行。

    def OnInit(self):
        window = wx.Frame(None, title='文件下载器', size=(400, 200))  # 修改窗口高度
        background = wx.Panel(window)
        self.user_input = user_input = wx.TextCtrl(background)

        submit = wx.Button(background, label='下载', size=(80, 25))
        submit.Bind(wx.EVT_BUTTON, self.fetchHandler)

        hbox = wx.BoxSizer()
        hbox.Add(user_input, proportion=1, flag=wx.ALL | wx.EXPAND, border=10)
        hbox.Add(submit, flag=wx.TOP | wx.BOTTOM | wx.RIGHT, border=10)

        self.files = files = wx.ListBox(background)  # 添加列表控件
        files.Bind(wx.EVT_LISTBOX_DCLICK, self.dclickItemHandler)  # 绑定列表项点击事件处理方法
        self.update_list()  # 更新文件列表

        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add(hbox, proportion=0, flag=wx.EXPAND)
        # 添加列表控件到垂直容器
        vbox.Add(files, proportion=1, flag=wx.EXPAND | wx.LEFT | wx.BOTTOM | wx.RIGHT, border=10)

        background.SetSizer(vbox)
        window.Show()
        return True

    def update_list(self):  # 定义更新文件列表的方法
        self.files.Set(self.server.file_list())  # 设置列表控件内容为文件列表

    def dclickItemHandler(self, event):  # 定义列表项点击事件的处理方法
        file_path = abspath('%s/%s' % (self.dir_name, self.files.GetStringSelection()))  # 获取文件绝对路径
        os.system(file_path)  # 打开文件

    def fetchHandler(self, event):
        filename = self.user_input.GetValue()
        try:
            self.server.fetch(filename, self.secret)
            self.update_list()  # 下载后更新文件列表
        except Fault as f:
            if f.faultCode != UNHANDLED:
                raise
            print('没有找到文件：', filename)

def main():
    url_file, dir_name, url = sys.argv[1:]
    client = Client(url_file, dir_name, url)
    client.MainLoop()

if __name__ == '__main__':
    main()
